--- chan_sip.c_orig	2008-06-03 23:46:24.000000000 +0900
+++ chan_sip.c	2008-07-06 00:58:30.000000000 +0900
@@ -1266,6 +1266,7 @@
 static void parse_moved_contact(struct sip_pvt *p, struct sip_request *req);
 static int sip_send_mwi_to_peer(struct sip_peer *peer);
 static int does_peer_need_mwi(struct sip_peer *peer);
+static int transmit_info_with_flash(struct sip_pvt *p);
 
 /*--- Dialog management */
 static struct sip_pvt *sip_alloc(ast_string_field callid, struct sockaddr_in *sin,
@@ -3971,6 +3972,9 @@
 	case AST_CONTROL_SRCUPDATE:
 		ast_rtp_new_source(p->rtp);
 		break;
+	case AST_CONTROL_FLASH:
+		transmit_info_with_flash(p);
+		break;
 	case -1:
 		res = -1;
 		break;
@@ -7836,6 +7840,18 @@
 	add_digit(&req, digit, duration);
 	return send_request(p, &req, XMIT_RELIABLE, p->ocseq);
 }
+/*! \brief Send SIP INFO FLASH message*/
+static int transmit_info_with_flash(struct sip_pvt *p)
+{
+	struct sip_request req;
+	char tmp[80];
+	reqprep(&req, p, SIP_INFO, 0, 1);
+	snprintf(tmp, sizeof(tmp), "event flashhook\r\n");
+	add_header(&req, "Content-Type", "application/broadsoft");
+	add_header_contentLength(&req, strlen(tmp));
+	add_line(&req, tmp);
+	return send_request(p, &req, XMIT_RELIABLE, p->ocseq);
+}
 
 /*! \brief Send SIP INFO with video update request */
 static int transmit_info_with_vidupdate(struct sip_pvt *p)
@@ -11189,6 +11205,27 @@
 	const char *c = get_header(req, "Content-Type");
 
 	/* Need to check the media/type */
+	if (!strcasecmp(c, "application/broadsoft")){
+		/* Try getting the "event" part */
+		strcpy(buf, req->line[0]);
+		if (ast_strlen_zero(buf)){
+			ast_log(LOG_WARNING, "Unable to retrieve DTMF signal from INFO message from %s\n", p->callid);
+			transmit_response(p, "200 OK", req); /* Should return error */
+			return;
+		}
+		if (!strcasecmp(buf, "event flashhook")){
+			/* send a FLASH event */
+			struct ast_frame f = { AST_FRAME_CONTROL, AST_CONTROL_FLASH, };
+			ast_queue_frame(p->owner, &f);
+			if (sipdebug)
+				ast_verbose("* dtmf event received: FLASH\n");								
+		} else {
+		ast_log(LOG_WARNING, "Unsupported DTMF: %s\n", buf);
+		}
+		transmit_response(p, "200 OK", req);
+		return;		
+	}
+	else 
 	if (!strcasecmp(c, "application/dtmf-relay") ||
 	    !strcasecmp(c, "application/vnd.nortelnetworks.digits")) {
 		unsigned int duration = 0;
